------------
-- SUMELM --
------------

module Sumelm where

import Auxiliary
import Expressions
import LPPE 
import MLPPE
import DataSpec

-- This function removes unnecessary summations from an LPPE.
sumelm :: PSpecification -> PSpecification
sumelm ((LPPE name pars summands), initialState, dataspec) = (LPPE name pars newSummands, initialState, dataspec)
  where
    newSummands = map (\x -> sumelmPSummand x (map fst (getLocalPars x))) summands

-- This function removes unnecessary summations from a summand.
sumelmPSummand :: PSummand -> [Variable] -> PSummand
sumelmPSummand s [] = s
sumelmPSummand (params, c, a, aps, probChoices, g) (var:vars) 
  | variableNotUsed = sumelmPSummand (paramsNew, c, a, aps, probChoices, g) vars
  | onlyOneValue    = sumelmPSummand (paramsNew, cNew, a, apsNew, probChoices, gNew) vars
  | otherwise       = sumelmPSummand (params, c, a, aps, probChoices, g) vars
  where
    paramsNew                = [(v,t) | (v,t) <- params, not (v == var)]
    variableNotUsed          = not(variableInExpression var c) && 
                               not(variableInExpressions var aps) &&
                               not(variableInExpressions var g)
    value                    = possibleValueForConditionToBeTrue var c
    onlyOneValue             = value /= Undecided
    UniqueValue uniqueChoice = value
    cNew                     = substituteInExpression [(var, uniqueChoice)] c
    apsNew                   = map (substituteInExpression [(var, uniqueChoice)]) aps
    gNew                     = if (elem var (map fst3 probChoices)) then g else (map (substituteInExpression [(var, uniqueChoice)]) g)

--------------------------

sumelmM :: MSpecification -> MSpecification
sumelmM ((MLPPE name pars summands), initialState, dataspec) = (MLPPE name pars newSummands, initialState, dataspec)
  where
    newSummands = map (\x -> sumelmSummandM x dataspec (getMLocalPars x)) summands

sumelmSummandM :: GeneralSummand -> DataSpec -> LocalPars -> GeneralSummand
sumelmSummandM s _ [] = s
sumelmSummandM (PSummand ps) _ lp = PSummand (sumelmPSummand ps (map fst lp))
sumelmSummandM (MSummand (params, c, lambda, g)) dataspec ((var,typ):vars)
  | variableNotUsed = sumelmSummandM (MSummand (paramsNew, c, Function "multiply" [Variable (show (length (getValues dataspec typ))), lambda], g)) dataspec vars
  | onlyOneValue    = sumelmSummandM (MSummand (paramsNew, cNew, lambdaNew, gNew)) dataspec vars
  | otherwise       = sumelmSummandM (MSummand (params, c, lambda, g)) dataspec vars
  where
    paramsNew                = [(v,t) | (v,t) <- params, not (v == var)]
    variableNotUsed          = not(variableInExpression var c) && 
                               not(variableInExpression var lambda) &&
                               not(variableInExpressions var g)
    value                    = possibleValueForConditionToBeTrue var c
    onlyOneValue             = value /= Undecided
    UniqueValue uniqueChoice = value
    cNew                     = substituteInExpression [(var, uniqueChoice)] c
    lambdaNew                = substituteInExpression [(var, uniqueChoice)] lambda
    gNew                     = map (substituteInExpression [(var, uniqueChoice)]) g